package lab10;

import java.awt.Image;
import java.util.ArrayList;
import java.util.List;

public class Animation {

    private final List<Image> images;
    private final List<Integer> durations;
    private int currentFrameIndex;
    private long currentFrameRemainingTime;
    private int startFrame;
    private int endFrame;
    private boolean repeating;
    private boolean forwards;
    private boolean running;

    public Animation() {
        images = new ArrayList();
        durations = new ArrayList();
        running = false;
    }

    public synchronized void addFrame(Image image, int duration) {
        images.add(image);
        durations.add(duration);
    }
    
    public void setRepeating(boolean repeating) {
        this.repeating = repeating;
    }
    
    public void setForwards(boolean forwards) {
        this.forwards = forwards;
    }

    public void setStartFrame(int startFrame) {
        if (startFrame >= 0 && startFrame < images.size()) {
            this.startFrame = startFrame;
        }
    }
    
    public void setEndFrame(int endFrame) {
        if (endFrame >= 0 && endFrame < images.size()) {
            this.endFrame = endFrame;
        }
    }

    public synchronized void start() {
        if (images.size() > 1) {
            currentFrameIndex = startFrame;
            currentFrameRemainingTime = durations.get(currentFrameIndex);
            running = true;
        }
    }
    
    public synchronized void stop() {
        running = false;
    }
    
    public synchronized boolean isRunning() {
    	return running;
    }
    
    public synchronized void update(long elapsedTime) {
        while (running && elapsedTime > 0) {
            elapsedTime -= currentFrameRemainingTime;
            currentFrameRemainingTime = -elapsedTime;
            if (elapsedTime >= 0) {
                if (currentFrameIndex==endFrame) {
                    if (repeating) {
                        currentFrameIndex = startFrame;
                    } else {
                        running = false;
                    }
                } else {
                    if (forwards) {
                        currentFrameIndex++;
                        if (currentFrameIndex >= images.size()) currentFrameIndex = 0;
                    } else {
                        currentFrameIndex--;
                        if (currentFrameIndex < 0) currentFrameIndex = images.size()-1;
                    }
                }
                currentFrameRemainingTime = durations.get(currentFrameIndex);
            }
        }
    }
    
    public synchronized Image getImage() {
        Image result = null;
        if (running) {
            result = images.get(currentFrameIndex);
        }
        return result;
    }
    
}