package lab10;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JPanel;


public class CardPanel extends JPanel implements ActionListener, MouseListener {
    
    private static final int CARD_WIDTH = 92;
    private static final int CARD_HEIGHT = 128;
    private static final int MARKER_WIDTH = 32;
    private static final int MARKER_HEIGHT = 32;
    
    private static final Image[] SELECT_MARKERS;
    private static final Image NO_CARD;
    static {
        SELECT_MARKERS = new Image[5];
        try {
            for(int i=0; i<5; i++) {
                SELECT_MARKERS[i] = ImageIO.read(new File("./images/Cards/Select" + (i+1) + ".png"));
            }
            NO_CARD = ImageIO.read(new File("./images/Cards/NoCard.png"));
        }
        catch (IOException e) {
            throw new ExceptionInInitializerError(e);
        }
    }
    
    private final JButton stopButton;
    private Card[] cards;
    private final List<Integer> selected;
    private boolean selecting;

    @SuppressWarnings("LeakingThisInConstructor")
    public CardPanel(JButton stopButton) {
        this.setPreferredSize(new Dimension(7*CARD_WIDTH,CARD_HEIGHT));
        this.stopButton = stopButton;
        this.cards = new Card[7];
        this.selected = new ArrayList();
        this.selecting = false;
        this.stopButton.addActionListener(this);
        this.addMouseListener(this);
    }
    
    public synchronized Card[] selectCards(Card[] cards) {
        stopButton.setEnabled(true);
        this.cards = cards;
        selected.clear();
        selecting = true;
        repaint();
        try {
            wait();
        } catch(InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        }
        Card[] result = new Card[selected.size()];
        for(int i=0; i<selected.size(); i++) {
            result[i] = cards[selected.get(i)];
        }
        return result;
    }

    @Override
    public void paintComponent(Graphics graphics) {
        for(int i=0; i<cards.length; i++) {
            if (cards[i] == null) {
                graphics.drawImage(NO_CARD,i*CARD_WIDTH,0,null);
            } else {
                graphics.drawImage(cards[i].getImage(),i*CARD_WIDTH,0,null);
            }
            int index = selected.indexOf(i);
            if (index != -1) {
                graphics.drawImage(SELECT_MARKERS[index],i*CARD_WIDTH + (CARD_WIDTH - MARKER_WIDTH) / 2,(CARD_HEIGHT - MARKER_HEIGHT) / 2,null);
            }
        }
    }
    
    @Override
    public synchronized void actionPerformed(ActionEvent e) {
        if (selected.size() == 5) {
            selecting = false;
            stopButton.setEnabled(false);
            notify();
        }
    }

    @Override
    public void mouseClicked(MouseEvent e) { }

    @Override
    public synchronized void mousePressed(MouseEvent e) { 
        if (selecting) {
            Integer cardIndex = e.getX() / CARD_WIDTH;  // using type Integer is important here so that remove below
            if (selected.contains(cardIndex)) {         // does not confuse elements in the list with indices.
                selected.remove(cardIndex);
            } else {
                if (selected.size() < 5) selected.add(cardIndex);
            }
            repaint();
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) { }

    @Override
    public void mouseEntered(MouseEvent e) { }

    @Override
    public void mouseExited(MouseEvent e) { }
    
}
