package lab10;

import java.awt.Graphics;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import static lab10.Direction.*;

public final class Robot {
    
    private static final int MOVE_DIST = 64;
    private static final float MOVE_SPEED = 0.1f;
    
    private static final Animation turnAnimation;
    private static final Animation idleAnimation;
    static {
        turnAnimation = new Animation();
        idleAnimation = new Animation();
        try {
            for(int i=0; i<88 ; i++) {
                if (i<72) {
                    turnAnimation.addFrame(ImageIO.read(new File("./images/Robot/Robot" + i + ".png")),50);
                } else {
                    idleAnimation.addFrame(ImageIO.read(new File("./images/Robot/Robot" + i + ".png")),600);
                }
            }
        } catch (IOException e) {
            throw new ExceptionInInitializerError(e);
        }
    }
    
    private Direction direction;
    private Animation currentAnimation;
    private float posX;
    private float posY;
    private float dX;
    private float dY;
    private boolean moving;
    private int goalX;
    private int goalY;
    
    public Robot(Direction direction, int posX, int posY) {
        this.direction = direction;
        setIdleAnimation();
        this.posX = posX;
        this.posY = posY;
        dX = 0;
        dY = 0;
        moving = false;
        goalX = 0;
        goalY = 0;
    }
    
    public synchronized void setTurn90Animation(boolean clockwise) {
        currentAnimation = turnAnimation;
        currentAnimation.setRepeating(false);
        currentAnimation.setForwards(clockwise);
        switch(direction) {
            case NORTH:
                currentAnimation.setStartFrame(0);
                if (clockwise) {
                    currentAnimation.setEndFrame(18);
                } else {
                    currentAnimation.setEndFrame(54);
                }
                break;
            case EAST:
                currentAnimation.setStartFrame(18);
                if (clockwise) {
                    currentAnimation.setEndFrame(36);
                } else {
                    currentAnimation.setEndFrame(0);
                }
                break;
            case SOUTH:
                currentAnimation.setStartFrame(36);
                if (clockwise) {
                    currentAnimation.setEndFrame(54);
                } else {
                    currentAnimation.setEndFrame(18);
                }
                break;
            case WEST:
                currentAnimation.setStartFrame(54);
                if (clockwise) {
                    currentAnimation.setEndFrame(0);
                } else {
                    currentAnimation.setEndFrame(36);
                }
                break;
        }
        currentAnimation.start();
    }
    
    public synchronized void setTurn180Animation() {
        currentAnimation = turnAnimation;
        currentAnimation.setRepeating(false);
        currentAnimation.setForwards(true);
        switch(direction) {
            case NORTH:
                currentAnimation.setStartFrame(0);
                currentAnimation.setEndFrame(36);
                break;
            case EAST:
                currentAnimation.setStartFrame(18);
                currentAnimation.setEndFrame(54);
                break;
            case SOUTH:
                currentAnimation.setStartFrame(36);
                currentAnimation.setEndFrame(0);
                break;
            case WEST:
                currentAnimation.setStartFrame(54);
                currentAnimation.setEndFrame(18);
                break;
        }
        currentAnimation.start();
    }
    
    public synchronized void setIdleAnimation() {
        currentAnimation = idleAnimation;
        currentAnimation.setRepeating(true);
        currentAnimation.setForwards(true);
        switch(direction) {
            case NORTH:
                currentAnimation.setStartFrame(0);
                currentAnimation.setEndFrame(3);
                break;
            case EAST:
                currentAnimation.setStartFrame(4);
                currentAnimation.setEndFrame(7);
                break;
            case SOUTH:
                currentAnimation.setStartFrame(8);
                currentAnimation.setEndFrame(11);
                break;
            case WEST:
                currentAnimation.setStartFrame(12);
                currentAnimation.setEndFrame(15);
                break;
        }
        currentAnimation.start();
    }
    
    public synchronized void update(long elapsedTime) {
        currentAnimation.update(elapsedTime);
        if (moving) {
            float oldDist = (goalX - posX) * (goalX - posX) + (goalY - posY) * (goalY - posY);  // saqrt not required for comparison
            posX += dX * elapsedTime;
            posY += dY * elapsedTime;
            float newDist = (goalX - posX) * (goalX - posX) + (goalY - posY) * (goalY - posY);
            if (oldDist < newDist) {
                posX = goalX;
                posY = goalY;
                moving = false;
                notify();
            }
        } else {
            if (!currentAnimation.isRunning()) {
                setIdleAnimation();
                notify();
            } 
        }
    }
    
    public synchronized void move(boolean forward) {
        setIdleAnimation();
        moving = true;
        switch(direction) {
            case NORTH:
                goalX = (int) posX;
                goalY = forward? (int) posY - MOVE_DIST : (int) posY + MOVE_DIST;
                dX = 0;
                dY = forward? -MOVE_SPEED : MOVE_SPEED;
                break;
            case EAST:
                goalX = forward? (int) posX + MOVE_DIST : (int) posX - MOVE_DIST;
                goalY = (int) posY;
                dX = forward? MOVE_SPEED : -MOVE_SPEED;
                dY = 0;
                break;
            case SOUTH:
                goalX = (int) posX;
                goalY = forward? (int) posY + MOVE_DIST : (int) posY - MOVE_DIST;
                dX = 0;
                dY = forward? MOVE_SPEED : -MOVE_SPEED;
                break;
            case WEST:
                goalX = forward? (int) posX - MOVE_DIST : (int) posX + MOVE_DIST;
                goalY = (int) posY;
                dX = forward? -MOVE_SPEED : MOVE_SPEED;
                dY = 0;
                break;
        }
        try {
            wait();
        } catch(InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        }
    }
    
    public synchronized void turn90(boolean clockwise) {
        setTurn90Animation(clockwise);
        direction = direction.turn90(clockwise);
        try {
            wait();
        } catch(InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        }
    }
    
    public synchronized void turn180() {
        setTurn180Animation();
        direction = direction.turn180();
        try {
            wait();
        } catch(InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        }
    }
    
    public synchronized void draw(Graphics graphics) {
        graphics.drawImage(currentAnimation.getImage(),(int)posX,(int)posY,null);
    }
    
}
