package lab10;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;

public class RobotCanvas extends Canvas implements Runnable {
    
    private final Thread thread;
    private final int pauseTime = 20;
    private boolean running;
    private Robot robot;
    
    public RobotCanvas() {
        thread = new Thread(this);
        running = false;
        setPreferredSize(new Dimension(0,600));
    }

    public void setRobot(Robot robot) {
        this.robot = robot;
    }
    
    public void start() {
        running = true;
        thread.start();
    }

    public void stop() {
        running = false;
        try {
            thread.join();
        } catch(InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        }
    }
    
    @Override
    @SuppressWarnings("SleepWhileInLoop")
    public void run() {
        createBufferStrategy(2);
        BufferStrategy strategy = getBufferStrategy(); 
        long lastLoopTime = System.currentTimeMillis();
        while (running) {
            long currentTime = System.currentTimeMillis();
            long delta = currentTime - lastLoopTime;
            lastLoopTime = currentTime;
            Graphics graphics = strategy.getDrawGraphics();
            graphics.clearRect(0,0,600,600);
            robot.update(delta);
            robot.draw(graphics);
            graphics.dispose();
            strategy.show();
            if (delta < pauseTime) {
                try { 
                    Thread.sleep(pauseTime-delta); 
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    throw new RuntimeException(e);
                }
            }
        }
    }
    
}
